bl_info = {
    "name": "ZipMerge",
    "blender": (3, 60, 0),
    "description": "Merge at least 3 verts between your mesh islands, select the vert you want to start merging from and run ZipWeld under the merge menu.",
    "location": "Edit Mode > Mesh > Merge",
    "version": (1, 1),
    "category": "Object",
    "author": "Nick Hodgson",
}

import bpy
import bmesh

class OBJECT_OT_zip_merge(bpy.types.Operator):
    bl_idname = "object.zip_merge"
    bl_label = "ZipMerge"
    bl_options = {'REGISTER', 'UNDO'}
    
    # Property to store the number of iterations
    iterations: bpy.props.IntProperty(
        name="Iterations",
        default=1,
        min=1,
        description="Number of iterations"
    )

    @staticmethod
    def growMergeVertex():
        
        # Get the active mesh object
        obj = bpy.context.object
        
        mesh = obj.data
        bm = bmesh.from_edit_mesh(mesh)
        bm.verts.ensure_lookup_table()

        # Check if the object is a mesh and is in edit mode
        if obj and obj.type == 'MESH' and obj.mode == 'EDIT':
            # Get the mesh data
            mesh = obj.data
            
            # Create a BMesh from the mesh data
            bm = bmesh.from_edit_mesh(mesh)
            
            # define start vertex
            start_vertex = [v.index for v in bm.verts if v.select]
            
            bpy.ops.mesh.select_more()
            
            # Get a list of selected vertices' indices
            expanded_verts = [v.index for v in bm.verts if v.select]
            
            bpy.ops.mesh.select_all(action='DESELECT')
            
            #define boundary vert list
            boundary_verts = []
           
            # Iterate through vertices and select boundary vertices
            for v in bm.verts:
                if v.is_boundary:
                    boundary_verts.append(v.index)
            
            common_elements = list(set(expanded_verts) & set(boundary_verts))
            
            # Check if there are more than 3 common elements
            #if len(common_elements) > 3:
                #raise Exception("Merge more verts between mesh and try again")
                #return  # Exit the function without performing the merge
            
            set_start_vertex = set(start_vertex)
            set_common_elements = set(common_elements)
            
            # Subtract elements of set2 from set1
            merge_verts = list(set_common_elements - set_start_vertex)
            
            # Select vertices from the list
            for v_idx in merge_verts:
                vertex = bm.verts[v_idx]
                vertex.select = True
                
            bpy.ops.mesh.merge(type='CENTER')
            
            # Ensure the index table is up-to-date
            bm.verts.ensure_lookup_table()

    def execute(self, context):
        # Get the number of iterations from the property
        num_iterations = self.iterations
        
        # Perform iterations
        for _ in range(num_iterations):
            # Call the growMergeVertex function
            self.growMergeVertex()
        return {'FINISHED'}

def menu_func(self, context):
    self.layout.operator(OBJECT_OT_zip_merge.bl_idname)

def register():
    bpy.utils.register_class(OBJECT_OT_zip_merge)
    bpy.types.VIEW3D_MT_edit_mesh_merge.append(menu_func)

def unregister():
    bpy.utils.unregister_class(OBJECT_OT_zip_merge)
    bpy.types.VIEW3D_MT_edit_mesh_merge.remove(menu_func)

if __name__ == "__main__":
    register()
